@@ -2,14 +2,18 @@ import base64
 import collections
 import contextlib
 import datetime
+import hashlib
+import itertools
 import json
 import logging
 import re
+import sys
 import time
 import zlib
 
 import elasticsearch
 
+from elasticsearch_raven import configuration
 from elasticsearch_raven import exceptions
 from elasticsearch_raven.postfix import postfix_encoded_data
 
@@ -48,28 +52,60 @@ class SentryMessage(collections.namedtuple('SentryMessage',
             raise exceptions.DamagedSentryMessageBodyError
 
 
-class ElasticsearchTransport:
-    def __init__(self, host, use_ssl=False):
-        self._host = host
-        self._use_ssl = use_ssl
+def get_configured_log_transport():
+    return LogTransport(configuration['host'],
+                        use_ssl=configuration['use_ssl'],
+                        http_auth=configuration['http_auth'])
 
-    def send(self, message):
+
+class LogTransport:
+    DOCUMENT_TYPE = 'raven-log'
+
+    def __init__(self, host, use_ssl=False, http_auth=None):
+        self._connection = elasticsearch.Elasticsearch(hosts=[host],
+                                                       http_auth=http_auth,
+                                                       use_ssl=use_ssl)
+
+    def send_message(self, message):
         message_body = message.decode_body()
         postfix_encoded_data(message_body)
-        dated_index = message_body['project'].format(datetime.datetime.now())
-        http_auth = '{}:{}'.format(message.headers['sentry_key'],
-                                   message.headers['sentry_secret'])
-        connection = elasticsearch.Elasticsearch(hosts=[self._host],
-                                                 http_auth=http_auth,
-                                                 use_ssl=self._use_ssl)
+        message_id = hash_dict(message_body)
+        index = message_body['project'].format(datetime.datetime.now())
+        self.send(message_body, index, message_id)
+
+    def send(self, body, index, message_id):
         for retry in retry_loop(15 * 60, delay=1, back_off=1.5):
             try:
                 with logger_level_to_error('elasticsearch'):
-                    connection.index(body=message_body, index=dated_index,
-                                     doc_type='raven-log')
+                    self._connection.index(body=body, index=index,
+                                           id=message_id,
+                                           doc_type=self.DOCUMENT_TYPE)
             except elasticsearch.exceptions.ConnectionError as e:
                 retry(e)
 
+    def search(self, segment_size=1000, **kwargs):
+        for offset in itertools.count(step=segment_size):
+            response = self._connection.search(doc_type=self.DOCUMENT_TYPE,
+                                               size=segment_size, from_=offset,
+                                               **kwargs)
+            hits = response['hits']['hits']
+            for hit in hits:
+                yield hit
+            if len(hits) < segment_size:
+                break
+
+    def delete(self, index, record_id):
+        self._connection.delete(index, self.DOCUMENT_TYPE, record_id)
+
+
+def hash_dict(dictionary):
+    message_json = json.dumps(
+        dictionary, indent=None, ensure_ascii=True, separators=(', ', ': '),
+        sort_keys=True)
+    sha1 = hashlib.sha1()
+    sha1.update(message_json.encode('ascii'))
+    return sha1.hexdigest()
+
 
 def retry_loop(timeout, delay, back_off=1.0):
     start_time = time.time()
@@ -99,3 +135,17 @@ def logger_level_to_error(logger_name):
     logger.setLevel(logging.ERROR)
     yield
     logger.setLevel(level)
+
+
+def update_ids():
+    log_transport = get_configured_log_transport()
+    all_count, modified_count = 0, 0
+    for log in log_transport.search():
+        all_count += 1
+        log_id = hash_dict(log['_source'])
+        if log['_id'] != log_id:
+            modified_count += 1
+            log_transport.send(log['_source'], log['_index'], log_id)
+            log_transport.delete(log['_index'], log['_id'])
+    sys.stdout.write('Logs: {}\nModified: {}\n'.format(all_count,
+                                                       modified_count))
